/*
* Copyright (c) 2005 X.Org Foundation L.L.C.
* 
* Permission is hereby granted, free of charge, to any person obtaining a copy of
* this software and associated documentation files (the "Software"), to deal in
* the Software without restriction, including without limitation the rights to
* use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
* of the Software, and to permit persons to whom the Software is furnished to do
* so, subject to the following conditions:
* 
* The above copyright notice and this permission notice shall be included in all
* copies or substantial portions of the Software.
* 
* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
* IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
* FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
* AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
* LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
* OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
* SOFTWARE.
* $Header: /cvs/xtest/xtest/xts5/tset/Xlib12/ptbckevnt/ptbckevnt.m,v 1.2 2005/11/03 08:42:36 jmichael Exp $
* 
* Copyright (c) Applied Testing and Technology, Inc. 1995
* All Rights Reserved.
* 
* 
* Portions of this software are based on Xlib and X Protocol Test Suite.
* We have used this material under the terms of its copyright, which grants
* free use, subject to the conditions below.  Note however that those
* portions of this software that are based on the original Test Suite have
* been significantly revised and that all such revisions are copyright (c)
* 1995 Applied Testing and Technology, Inc.  Insomuch as the proprietary
* revisions cannot be separated from the freely copyable material, the net
* result is that use of this software is governed by the ApTest copyright.
* 
* Copyright (c) 1990, 1991  X Consortium
* 
* Permission is hereby granted, free of charge, to any person obtaining a copy
* of this software and associated documentation files (the "Software"), to deal
* in the Software without restriction, including without limitation the rights
* to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
* copies of the Software, and to permit persons to whom the Software is
* furnished to do so, subject to the following conditions:
* 
* The above copyright notice and this permission notice shall be included in
* all copies or substantial portions of the Software.
* 
* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
* IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
* FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL THE
* X CONSORTIUM BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN
* AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
* CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
* 
* Except as contained in this notice, the name of the X Consortium shall not be
* used in advertising or otherwise to promote the sale, use or other dealings
* in this Software without prior written authorization from the X Consortium.
* 
* Permission to use, copy, modify, distribute, and sell this software and
* its documentation for any purpose is hereby granted without fee,
* provided that the above copyright notice appear in all copies and that
* both that copyright notice and this permission notice appear in
* supporting documentation, and that the name of UniSoft not be
* used in advertising or publicity pertaining to distribution of the
* software without specific, written prior permission.  UniSoft
* makes no representations about the suitability of this software for any
* purpose.  It is provided "as is" without express or implied warranty.
* 
*/
/*
 * SYNOPSIS:
 *   void
 *   XPutBackEvent(display, event)
 *   Display *display;
 *   XEvent	*event;
 */

/* $Header: /cvs/xtest/xtest/xts5/lib/mc/mcinclude.mc,v 1.2 2005/02/12 17:50:23 anderson Exp $ */

#include	<stdlib.h>
#include	"xtest.h"
#include	"X11/Xlib.h"
#include	"X11/Xutil.h"
#include	"X11/Xresource.h"
#include	"X11/keysym.h"
#include	"tet_api.h"
#include	"xtestlib.h"
#include	"pixval.h"
#ifdef INPUTEXTENSION
#include        "X11/extensions/XInput.h"
#include        "XItest.h"
#endif
/* $Header: /cvs/xtest/xtest/xts5/lib/mc/mcextern.mc,v 1.1.1.1 2005/02/12 14:37:14 anderson Exp $ */

extern	Display	*Dsp;
extern	Window	Win;

extern	Window	ErrdefWindow;
extern	Drawable ErrdefDrawable;
extern	GC		ErrdefGC;
extern	Colormap ErrdefColormap;
extern	Pixmap	ErrdefPixmap;
extern	Atom	ErrdefAtom;
extern	Cursor	ErrdefCursor;
extern	Font	ErrdefFont;


#define T_XPutBackEvent	1
char    *TestName = "XPutBackEvent";

/*
 * Defines for different argument types
 */
#define A_DISPLAY display


/*
 * Arguments to the XPutBackEvent function
 */
static Display *display;
static XEvent	*event;


/*
 * Can not use "xcall" because it empties the event queue.
 */
#define	_xcall_()	\
		_startcall(display);\
		XPutBackEvent(display, event);\
		_endcall(display)
static XEvent _event;

int 	tet_thistest;

/*
 * Called at the beginning of each test purpose to reset the
 * arguments to their initial values
 */
static void
setargs()
{
	display = Dsp;
	event = &_event;
}

/*
 * Set the arguments to default values for error tests
 */
static void
seterrdef()
{
}

static void t001(){

XEvent	event_return;
int 	pass = 0, fail = 0;

 	report_purpose(1);

	report_assertion("Assertion XPutBackEvent-1.(A)");
	report_assertion("A call to XPutBackEvent pushes a copy of event onto the");
	report_assertion("head of the display's event queue.");

	report_strategy("Call XSync to empty event queue.");
	report_strategy("Call XPutBackEvent to push event onto the head of the event queue.");
	report_strategy("Call XPeekEvent to verify that first event is the event that was pushed.");
	report_strategy("Call XPutBackEvent to push another event onto the head of the event queue.");
	report_strategy("Call XPeekEvent to verify that first event is the event that was pushed.");

	tpstartup();
	setargs();
/* Call XSync to empty event queue. */
	XSync(display, True);
/* Call XPutBackEvent to push event onto the head of the event queue. */
	event->type = ButtonPress;
	_xcall_();
/* Call XPeekEvent to verify that first event is the event that was pushed. */
	XPeekEvent(display, &event_return);
	if (event_return.type != event->type) {
		report("Returned %s, expected %s", eventname(event_return.type), eventname(event->type));
		FAIL;
	}
	else
		CHECK;
/* Call XPutBackEvent to push another event onto the head of the event queue. */
	event->type = KeyPress;
	_xcall_();
/* Call XPeekEvent to verify that first event is the event that was pushed. */
	XPeekEvent(display, &event_return);
	if (event_return.type != event->type) {
		report("Returned %s, expected %s", eventname(event_return.type), eventname(event->type));
		FAIL;
	}
	else
		CHECK;
	XSync(display, True);
	
	CHECKPASS(2);
	tpcleanup();
	pfcount(pass, fail);
}

static void t002(){

int	i;
int 	pass = 0, fail = 0;

 	report_purpose(2);

	report_assertion("Assertion XPutBackEvent-2.(B)");
	report_assertion("A call to XPutBackEvent can be made an unlimited number of");
	report_assertion("times in succession.");
	report_assertion("Reason for omission: Testing the assertion would require an unreasonable amount of time or resources on most systems");

	report_strategy("Call XPutBackEvent 1000 times.");

	tpstartup();
	setargs();
/* Call XPutBackEvent 1000 times. */
	event->type = ButtonPress;
	for (i=0; i<1000; i++) {
		_xcall_();
	}
	UNTESTED;
	tpcleanup();
	pfcount(pass, fail);
}

/* End of Test Cases */


struct tet_testlist tet_testlist[] = {
	t001, 1,
	t002, 2,
	NULL, 0
};

int 	ntests = sizeof(tet_testlist)/sizeof(struct tet_testlist)-1;

void	(*tet_startup)() = startup;
void	(*tet_cleanup)() = cleanup;
