/*
* Copyright (c) 2005 X.Org Foundation L.L.C.
* 
* Permission is hereby granted, free of charge, to any person obtaining a copy of
* this software and associated documentation files (the "Software"), to deal in
* the Software without restriction, including without limitation the rights to
* use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
* of the Software, and to permit persons to whom the Software is furnished to do
* so, subject to the following conditions:
* 
* The above copyright notice and this permission notice shall be included in all
* copies or substantial portions of the Software.
* 
* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
* IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
* FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
* AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
* LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
* OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
* SOFTWARE.
* $Header: /cvs/xtest/xtest/xts5/tset/Xlib17/clpbx/clpbx.m,v 1.2 2005/11/03 08:43:03 jmichael Exp $
* 
* Copyright (c) Applied Testing and Technology, Inc. 1995
* All Rights Reserved.
* 
* 
* Portions of this software are based on Xlib and X Protocol Test Suite.
* We have used this material under the terms of its copyright, which grants
* free use, subject to the conditions below.  Note however that those
* portions of this software that are based on the original Test Suite have
* been significantly revised and that all such revisions are copyright (c)
* 1995 Applied Testing and Technology, Inc.  Insomuch as the proprietary
* revisions cannot be separated from the freely copyable material, the net
* result is that use of this software is governed by the ApTest copyright.
* 
* Copyright (c) 1990, 1991  X Consortium
* 
* Permission is hereby granted, free of charge, to any person obtaining a copy
* of this software and associated documentation files (the "Software"), to deal
* in the Software without restriction, including without limitation the rights
* to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
* copies of the Software, and to permit persons to whom the Software is
* furnished to do so, subject to the following conditions:
* 
* The above copyright notice and this permission notice shall be included in
* all copies or substantial portions of the Software.
* 
* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
* IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
* FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL THE
* X CONSORTIUM BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN
* AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
* CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
* 
* Except as contained in this notice, the name of the X Consortium shall not be
* used in advertising or otherwise to promote the sale, use or other dealings
* in this Software without prior written authorization from the X Consortium.
* 
* Permission to use, copy, modify, distribute, and sell this software and
* its documentation for any purpose is hereby granted without fee,
* provided that the above copyright notice appear in all copies and that
* both that copyright notice and this permission notice appear in
* supporting documentation, and that the name of UniSoft not be
* used in advertising or publicity pertaining to distribution of the
* software without specific, written prior permission.  UniSoft
* makes no representations about the suitability of this software for any
* purpose.  It is provided "as is" without express or implied warranty.
* 
*/
/*
 * SYNOPSIS:
 *   
 *   XClipBox(r, rect_return)
 *   Region		r;
 *   XRectangle	*rect_return;
 */

/* $Header: /cvs/xtest/xtest/xts5/lib/mc/mcinclude.mc,v 1.2 2005/02/12 17:50:23 anderson Exp $ */

#include	<stdlib.h>
#include	"xtest.h"
#include	"X11/Xlib.h"
#include	"X11/Xutil.h"
#include	"X11/Xresource.h"
#include	"X11/keysym.h"
#include	"tet_api.h"
#include	"xtestlib.h"
#include	"pixval.h"
#ifdef INPUTEXTENSION
#include        "X11/extensions/XInput.h"
#include        "XItest.h"
#endif
/* $Header: /cvs/xtest/xtest/xts5/lib/mc/mcextern.mc,v 1.1.1.1 2005/02/12 14:37:14 anderson Exp $ */

extern	Display	*Dsp;
extern	Window	Win;

extern	Window	ErrdefWindow;
extern	Drawable ErrdefDrawable;
extern	GC		ErrdefGC;
extern	Colormap ErrdefColormap;
extern	Pixmap	ErrdefPixmap;
extern	Atom	ErrdefAtom;
extern	Cursor	ErrdefCursor;
extern	Font	ErrdefFont;


#define T_XClipBox	1
char    *TestName = "XClipBox";

/*
 * Defines for different argument types
 */


/*
 * Arguments to the XClipBox function
 */
static Region		r;
static XRectangle	*rect_return;


Status
checknotclear(disp, d, fg)
Display		*disp;
Drawable	d;
unsigned long	fg;
{
XImage			*im;
unsigned long		pix;
unsigned int		width;
unsigned int		height;
int			x;
int			y;

	getsize(Dsp, d, &width, &height);
	
        if ((im = XGetImage(disp, d, 0, 0, width, height, AllPlanes, ZPixmap)) == (XImage*) NULL) {
                delete("XGetImage failed");
                return(False);
        }

	for(y=0; y<height; y++)		
		for(x=0; x<width; x++)
			if(XGetPixel(im, x, y) == fg)
				return(True);
	return(False);
}

int 	tet_thistest;

/*
 * Called at the beginning of each test purpose to reset the
 * arguments to their initial values
 */
static void
setargs()
{
	r = 0;
	rect_return = 0;
}

/*
 * Set the arguments to default values for error tests
 */
static void
seterrdef()
{
}

static void t001(){

static XPoint   points[] = { {10, 10},  {74, 10},  {75, 30},  {65, 60},  {65, 30}, {75, 60},
                         {30, 50},  {30, 74},  {60, 74},   {60, 5},   {70, 5},  {70, 75}, {10, 85} };
Region		reg;
XRectangle	rect;
XVisualInfo	*vi;
Window		win;
GC		gc;
GC		gc2;
unsigned int		width;
unsigned int		height;
int 	pass = 0, fail = 0;

 	report_purpose(1);

	report_assertion("Assertion XClipBox-1.(A)");
	report_assertion("A call to XClipBox returns in the rect_return argument the");
	report_assertion("smallest rectangle enclosing the region r.");

	report_strategy("Create a region using XPolygonRegion.");
	report_strategy("Create a gc using XCreateGC.");
	report_strategy("Create a window using XCreateWindow.");
	report_strategy("Set the clip mask of the gc to the region.");
	report_strategy("Fill the entire drawable with W_FG using XFillRectangle.");
	report_strategy("Verify that the drawable is not completely W_BG.");
	report_strategy("Set the foreground of the gc to W_BG.");
	report_strategy("Obtain the smallest rectangle enclosing the polygon using XClipBox.");
	report_strategy("Draw the returned rectangle in W_BG using XFillRectangle.");
	report_strategy("Verify that the drawable is completely W_BG");
	report_strategy("");
	report_strategy("Create a window using XCreateWindow.");
	report_strategy("Fill the entire drawable with W_FG using XFillRectangle and the first gc.");
	report_strategy("Verify that the drawable is not completely W_BG.");
	report_strategy("Draw the smallest enclosing rectangle offset by +1+0 in W_BG using XFillRectangle and the second gc.");
	report_strategy("Verify that the drawable is not completely W_BG.");
	report_strategy("");
	report_strategy("Create a window using XCreateWindow.");
	report_strategy("Fill the entire drawable with W_FG using XFillRectangle and the first gc.");
	report_strategy("Verify that the drawable is not completely W_BG.");
	report_strategy("Draw the smallest enclosing rectangle offset by -1+0 in W_BG using XFillRectangle and the second gc.");
	report_strategy("Verify that the drawable is not completely W_BG.");
	report_strategy("");
	report_strategy("Create a window using XCreateWindow.");
	report_strategy("Fill the entire drawable with W_FG using XFillRectangle and the first gc.");
	report_strategy("Verify that the drawable is not completely W_BG.");
	report_strategy("Draw the smallest enclosing rectangle offset by +0+1 in W_BG using XFillRectangle and the second gc.");
	report_strategy("Verify that the drawable is not completely W_BG.");
	report_strategy("");
	report_strategy("Create a window using XCreateWindow.");
	report_strategy("Fill the entire drawable with W_FG using XFillRectangle and the first gc.");
	report_strategy("Verify that the drawable is not completely W_BG.");
	report_strategy("Draw the smallest enclosing rectangle offset by +0-1 in W_BG using XFillRectangle and the second gc.");
	report_strategy("Verify that the drawable is not completely W_BG.");

	tpstartup();
	setargs();
	reg = XPolygonRegion( points, NELEM(points), WindingRule);
	resetvinf(VI_WIN);
	nextvinf(&vi);
	win = makewin(Dsp, vi);
	gc = makegc(Dsp, win);
	gc2 = makegc(Dsp, win);
	XSetRegion(Dsp, gc, reg);
	XSetForeground(Dsp, gc, W_FG);
	XSetForeground(Dsp, gc2, W_BG);
	getsize(Dsp, win, &width, &height);
	XFillRectangle(Dsp, win, gc, 0,0, width, height);	

	if(checknotclear(Dsp, win, W_FG) == 0) {
		delete("XFillRectangle did not set any pixel to foreground.");
		return;
	} else
		CHECK;

	r = reg;
	rect_return = &rect;
	startcall(Dsp);
	if (isdeleted())
		return;
	XClipBox(r, rect_return);
	endcall(Dsp);
	if (geterr() != Success) {
		report("Got %s, Expecting Success", errorname(geterr()));
		FAIL;
	}

	XDestroyRegion(reg);

	XFillRectangle(Dsp, win, gc2, rect.x, rect.y, rect.width, rect.height);	

	if(checknotclear(Dsp, win, W_FG) != 0 ) {
		report("With a region used as the clip-mask in a GC,");
		report("%s() did not return a sufficiently large rectangle.", TestName);
		FAIL;
		return;
	} else
		CHECK;

	win = makewin(Dsp, vi);
	XFillRectangle(Dsp, win, gc, 0,0, width, height);	

	if(checknotclear(Dsp, win, W_FG) == 0) {
		delete("XFillRectangle did not set any pixel to foreground.");
		return;
	} else
		CHECK;
	XFillRectangle(Dsp, win, gc2, rect.x+1, rect.y, rect.width, rect.height);	

	if(checknotclear(Dsp, win, W_FG) == 0 ) {
		report("With a region used as the clip-mask in a GC,");
		report("%s() did not return a sufficiently small rectangle.in the x-direction.", TestName);
		FAIL;
	} else
		CHECK;

	win = makewin(Dsp, vi);
	XFillRectangle(Dsp, win, gc, 0,0, width, height);	

	if(checknotclear(Dsp, win, W_FG) == 0) {
		delete("XFillRectangle did not set any pixel to foreground.");
		return;
	} else
		CHECK;
	XFillRectangle(Dsp, win, gc2, rect.x-1, rect.y, rect.width, rect.height);	

	if(checknotclear(Dsp, win, W_FG) == 0 ) {
		report("With a region used as the clip-mask in a GC,");
		report("%s() did not return a sufficiently small rectangle in the x-direction.", TestName);
		FAIL;
	} else
		CHECK;

	win = makewin(Dsp, vi);
	XFillRectangle(Dsp, win, gc, 0,0, width, height);	

	if(checknotclear(Dsp, win, W_FG) == 0) {
		delete("XFillRectangle did not set any pixel to foreground.");
		return;
	} else
		CHECK;

	XFillRectangle(Dsp, win, gc2, rect.x, rect.y+1, rect.width, rect.height);	

	if(checknotclear(Dsp, win, W_FG) == 0 ) {
		report("With a region used as the clip-mask in a GC,");
		report("%s() did not return a sufficiently small rectangle in the y-direction.", TestName);
		FAIL;
	} else
		CHECK;

	win = makewin(Dsp, vi);
	XFillRectangle(Dsp, win, gc, 0,0, width, height);	

	if(checknotclear(Dsp, win, W_FG) == 0) {
		delete("XFillRectangle did not set any pixel to foreground.");
		return;
	} else
		CHECK;

	XFillRectangle(Dsp, win, gc2, rect.x, rect.y-1, rect.width, rect.height);	

	if(checknotclear(Dsp, win, W_FG) == 0 ) {
		report("With a region used as the clip-mask in a GC,");
		report("%s() did not return a sufficiently small rectangle in the y-direction.", TestName);
		FAIL;
	} else
		CHECK;

	CHECKPASS(10);
	tpcleanup();
	pfcount(pass, fail);
}

/* End of Test Cases */


struct tet_testlist tet_testlist[] = {
	t001, 1,
	NULL, 0
};

int 	ntests = sizeof(tet_testlist)/sizeof(struct tet_testlist)-1;

void	(*tet_startup)() = startup;
void	(*tet_cleanup)() = cleanup;
