/*
* Copyright (c) 2005 X.Org Foundation L.L.C.
* 
* Permission is hereby granted, free of charge, to any person obtaining a copy of
* this software and associated documentation files (the "Software"), to deal in
* the Software without restriction, including without limitation the rights to
* use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
* of the Software, and to permit persons to whom the Software is furnished to do
* so, subject to the following conditions:
* 
* The above copyright notice and this permission notice shall be included in all
* copies or substantial portions of the Software.
* 
* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
* IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
* FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
* AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
* LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
* OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
* SOFTWARE.
* $Header: /cvs/xtest/xtest/xts5/tset/Xlib17/lkpkysym/lkpkysym.m,v 1.2 2005/11/03 08:43:08 jmichael Exp $
* 
* Copyright (c) Applied Testing and Technology, Inc. 1995
* All Rights Reserved.
* 
* 
* Portions of this software are based on Xlib and X Protocol Test Suite.
* We have used this material under the terms of its copyright, which grants
* free use, subject to the conditions below.  Note however that those
* portions of this software that are based on the original Test Suite have
* been significantly revised and that all such revisions are copyright (c)
* 1995 Applied Testing and Technology, Inc.  Insomuch as the proprietary
* revisions cannot be separated from the freely copyable material, the net
* result is that use of this software is governed by the ApTest copyright.
* 
* Copyright (c) 1990, 1991  X Consortium
* 
* Permission is hereby granted, free of charge, to any person obtaining a copy
* of this software and associated documentation files (the "Software"), to deal
* in the Software without restriction, including without limitation the rights
* to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
* copies of the Software, and to permit persons to whom the Software is
* furnished to do so, subject to the following conditions:
* 
* The above copyright notice and this permission notice shall be included in
* all copies or substantial portions of the Software.
* 
* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
* IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
* FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL THE
* X CONSORTIUM BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN
* AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
* CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
* 
* Except as contained in this notice, the name of the X Consortium shall not be
* used in advertising or otherwise to promote the sale, use or other dealings
* in this Software without prior written authorization from the X Consortium.
* 
* Permission to use, copy, modify, distribute, and sell this software and
* its documentation for any purpose is hereby granted without fee,
* provided that the above copyright notice appear in all copies and that
* both that copyright notice and this permission notice appear in
* supporting documentation, and that the name of UniSoft not be
* used in advertising or publicity pertaining to distribution of the
* software without specific, written prior permission.  UniSoft
* makes no representations about the suitability of this software for any
* purpose.  It is provided "as is" without express or implied warranty.
* 
*/
/*
 * SYNOPSIS:
 *   KeySym
 *   XLookupKeysym(key_event, index)
 *   XKeyEvent	*key_event;
 *   int		index;
 */

/* $Header: /cvs/xtest/xtest/xts5/lib/mc/mcinclude.mc,v 1.2 2005/02/12 17:50:23 anderson Exp $ */

#include	<stdlib.h>
#include	"xtest.h"
#include	"X11/Xlib.h"
#include	"X11/Xutil.h"
#include	"X11/Xresource.h"
#include	"X11/keysym.h"
#include	"tet_api.h"
#include	"xtestlib.h"
#include	"pixval.h"
#ifdef INPUTEXTENSION
#include        "X11/extensions/XInput.h"
#include        "XItest.h"
#endif
/* $Header: /cvs/xtest/xtest/xts5/lib/mc/mcextern.mc,v 1.1.1.1 2005/02/12 14:37:14 anderson Exp $ */

extern	Display	*Dsp;
extern	Window	Win;

extern	Window	ErrdefWindow;
extern	Drawable ErrdefDrawable;
extern	GC		ErrdefGC;
extern	Colormap ErrdefColormap;
extern	Pixmap	ErrdefPixmap;
extern	Atom	ErrdefAtom;
extern	Cursor	ErrdefCursor;
extern	Font	ErrdefFont;


#define T_XLookupKeysym	1
char    *TestName = "XLookupKeysym";

/*
 * Defines for different argument types
 */


/*
 * Arguments to the XLookupKeysym function
 */
static XKeyEvent	*key_event;
static int		index;


int 	tet_thistest;

/*
 * Called at the beginning of each test purpose to reset the
 * arguments to their initial values
 */
static void
setargs()
{
	key_event = 0;
	index = 0;
}

/*
 * Set the arguments to default values for error tests
 */
static void
seterrdef()
{
}

static void t001(){

int		minkc, maxkc;
int		syms_per_kc;
KeyCode		kc;
KeySym		ks, res;
KeySym		*keysyms;
XKeyEvent	ke;
int		i;
int 	pass = 0, fail = 0;

 	report_purpose(1);

	report_assertion("Assertion XLookupKeysym-1.(A)");
	report_assertion("A call to XLookupKeysym returns the KeySym element");
	report_assertion("specified by index of the KeyCode vector specified by the");
	report_assertion("keycode member of the key_event argument.");

	report_strategy("Obtain the maximum and minimum KeyCodes using XDisplayKeycodes.");
	report_strategy("Obtain the number of KeySyms per KeyCode using XGetKeyboardMapping.");
	report_strategy("For each KeyCode and index:");
	report_strategy("   Obtain the corresponding KeySym using XKeycodeToKeysym.");
	report_strategy("   Obtain the corresponding KeySym using XLookupKeysym.");
	report_strategy("   Verify that the KeySyms are identical.");
	report_strategy("");

	tpstartup();
	setargs();
	XDisplayKeycodes(Dsp, &minkc, &maxkc);
	keysyms = XGetKeyboardMapping(Dsp, minkc, 1, &syms_per_kc);
	XFree((char *) keysyms);
	key_event = &ke;
	ke.display = Dsp;

	for(i=minkc; i<=maxkc; i++)
		for(index = 0; index < syms_per_kc; index++) {
			ks = XKeycodeToKeysym(Dsp, i, index);
			ke.keycode = i;
			startcall(Dsp);
			if (isdeleted())
				return;
			res = XLookupKeysym(key_event, index);
			endcall(Dsp);
			if (geterr() != Success) {
				report("Got %s, Expecting Success", errorname(geterr()));
				FAIL;
			}
			if(res != ks){
				report("%s() returned KeySym %lu instead of %lu for KeyCode %lu with index %d.", TestName, (long) res, (long) ks, (long) i, index);
				FAIL;
			} else
				CHECK;
		}


	CHECKPASS( (1+maxkc-minkc) * syms_per_kc);

	tpcleanup();
	pfcount(pass, fail);
}

static void t002(){

int		minkc, maxkc;
int		syms_per_kc;
KeyCode		kc;
KeySym		res;
KeySym		*keysyms;
XKeyEvent	ke;
int 	pass = 0, fail = 0;

 	report_purpose(2);

	report_assertion("Assertion XLookupKeysym-2.(A)");
	report_assertion("When no KeySym is defined for the keycode of the event");
	report_assertion("key_event, then a call to XLookupKeysym returns NoSymbol.");

	report_strategy("Obtain the maximum and minimum KeyCodes using XDisplayKeycodes.");
	report_strategy("Obtain the number of KeySyms per KeyCode using XGetKeyboardMapping.");
	report_strategy("For each index:");
	report_strategy("   Obtain the KeySym for a KeyCode greater than maximum KeyCode using XLookupKeysym.");
	report_strategy("   Verify that the KeySym returned is NoSymbol.");

	tpstartup();
	setargs();
	XDisplayKeycodes(Dsp, &minkc, &maxkc);
	keysyms = XGetKeyboardMapping(Dsp, minkc, 1, &syms_per_kc);
	XFree( (char *) keysyms);
	kc = maxkc + 1;

	if(kc == minkc) {
		delete("Cannot establish an invalid KeyCode.");
		return;
	} else
		CHECK;

	key_event = &ke;
	ke.display = Dsp;
	ke.keycode = kc;

	for(index=0; index < syms_per_kc; index++) {

		startcall(Dsp);
		if (isdeleted())
			return;
		res = XLookupKeysym(key_event, index);
		endcall(Dsp);
		if (geterr() != Success) {
			report("Got %s, Expecting Success", errorname(geterr()));
			FAIL;
		}

		if(res != NoSymbol) {
			report("%s() returned %lu instead of NoSymbol (%lu) for invalid KeyCode %lu with index %d.", TestName, (long) res, (long) NoSymbol, (long) kc, index);
			FAIL;
		} else
			CHECK;
	}

	CHECKPASS(1 + syms_per_kc);
	tpcleanup();
	pfcount(pass, fail);
}

/* End of Test Cases */


struct tet_testlist tet_testlist[] = {
	t001, 1,
	t002, 2,
	NULL, 0
};

int 	ntests = sizeof(tet_testlist)/sizeof(struct tet_testlist)-1;

void	(*tet_startup)() = startup;
void	(*tet_cleanup)() = cleanup;
